@@ -9,7 +9,7 @@
  * Biological Structures at Stanford, funded under the NIH Roadmap for        *
  * Medical Research, grant U54 GM072970. See https://simtk.org.               *
  *                                                                            *
- * Portions copyright (c) 2008-2021 Stanford University and the Authors.      *
+ * Portions copyright (c) 2008-2022 Stanford University and the Authors.      *
  * Authors: Peter Eastman                                                     *
  * Contributors:                                                              *
  *                                                                            *
@@ -88,7 +88,27 @@ namespace OpenMM {
  * above example, the energy only depends on the products sigma1*sigma2 and epsilon1*epsilon2, both of which are unchanged
  * if the labels 1 and 2 are reversed.  In contrast, if it depended on the difference sigma1-sigma2, the results would
  * be undefined, because reversing the labels 1 and 2 would change the energy.
+ * 
+ * The energy also may depend on "computed values".  These are similar to per-particle parameters, but instead of being
+ * specified in advance, their values are computed based on global and per-particle parameters.  For example, the following
+ * code uses a global parameter (lambda) to interpolate between two different sigma values for each particle (sigmaA and sigmaB).
+ * 
+ * \verbatim embed:rst:leading-asterisk
+ * .. code-block:: cpp
  *
+ *    CustomNonbondedForce* force = new CustomNonbondedForce("4*epsilon*((sigma/r)^12-(sigma/r)^6); sigma=0.5*(sigma1+sigma2); epsilon=sqrt(epsilon1*epsilon2)");
+ *    force->addComputedValue("sigma", "(1-lambda)*sigmaA + lambda*sigmaB");
+ *    force->addGlobalParameter("lambda", 0);
+ *    force->addPerParticleParameter("sigmaA");
+ *    force->addPerParticleParameter("sigmaB");
+ *    force->addPerParticleParameter("epsilon");
+ *
+ * \endverbatim
+ *
+ * You could, of course, embed the computation of sigma directly into the energy expression, but then it would need to be
+ * repeated for every interaction.  By separating it out as a computed value, it only needs to be computed once for each
+ * particle instead of once for each interaction, thus saving computation time.
+ * 
  * CustomNonbondedForce can operate in two modes.  By default, it computes the interaction of every particle in the System
  * with every other particle.  Alternatively, you can restrict it to only a subset of particle pairs.  To do this, specify
  * one or more "interaction groups".  An interaction group consists of two sets of particles that should interact with
@@ -210,6 +230,12 @@ public:
     int getNumFunctions() const {
         return functions.size();
     }
+    /**
+     * Get the number of per-particle computed values the interaction depends on.
+     */
+    int getNumComputedValues() const {
+        return computedValues.size();
+    }
     /**
      * Get the number of interaction groups that have been defined.
      */
@@ -465,6 +491,36 @@ public:
      * If the specified function is not a Continuous1DFunction, this throws an exception.
      */
     void setFunctionParameters(int index, const std::string& name, const std::vector<double>& values, double min, double max);
+    /**
+     * Add a computed value to calculate for each particle.
+     *
+     * @param name        the name of the value
+     * @param expression  an algebraic expression to evaluate when calculating the computed value.  It may
+     *                    depend on the values of per-particle and global parameters, but not one other
+     *                    computed values.
+     * @return the index of the computed value that was added
+     */
+    int addComputedValue(const std::string& name, const std::string& expression);
+    /**
+     * Get the properties of a computed value.
+     *
+     * @param index            the index of the computed value for which to get parameters
+     * @param[out] name        the name of the value
+     * @param[out] expression  an algebraic expression to evaluate when calculating the computed value.  It may
+     *                         depend on the values of per-particle and global parameters, but not one other
+     *                         computed values.
+     */
+    void getComputedValueParameters(int index, std::string& name, std::string& expression) const;
+    /**
+     * Set the properties of a computed value.
+     *
+     * @param index       the index of the computed value for which to set parameters
+     * @param name        the name of the value
+     * @param expression  an algebraic expression to evaluate when calculating the computed value.  It may
+     *                    depend on the values of per-particle and global parameters, but not one other
+     *                    computed values.
+     */
+    void setComputedValueParameters(int index, const std::string& name, const std::string& expression);
     /**
      * Add an interaction group.  An interaction will be computed between every particle in set1 and every particle in set2.
      *
@@ -520,6 +576,7 @@ private:
     class GlobalParameterInfo;
     class ExclusionInfo;
     class FunctionInfo;
+    class ComputedValueInfo;
     class InteractionGroupInfo;
     NonbondedMethod nonbondedMethod;
     double cutoffDistance, switchingDistance;
@@ -530,6 +587,7 @@ private:
     std::vector<ParticleInfo> particles;
     std::vector<ExclusionInfo> exclusions;
     std::vector<FunctionInfo> functions;
+    std::vector<ComputedValueInfo> computedValues;
     std::vector<InteractionGroupInfo> interactionGroups;
     std::vector<int> energyParameterDerivatives;
 };
@@ -603,6 +661,19 @@ public:
     }
 };
 
+/**
+ * This is an internal class used to record information about a computed value.
+ * @private
+ */
+class CustomNonbondedForce::ComputedValueInfo {
+public:
+    std::string name, expression;
+    ComputedValueInfo() {
+    }
+    ComputedValueInfo(const std::string& name, const std::string& expression) : name(name), expression(expression) {
+    }
+};
+
 /**
  * This is an internal class used to record information about an interaction group.
  * @private
