@@ -32,6 +32,7 @@
 #include "openmm/OpenMMException.h"
 #include "openmm/internal/GBSAOBCForceImpl.h"
 #include "openmm/internal/ContextImpl.h"
+#include "openmm/internal/Messages.h"
 #include "openmm/kernels.h"
 #include <vector>
 
@@ -50,7 +51,7 @@ void GBSAOBCForceImpl::initialize(ContextImpl& context) {
         context.getSystem().getDefaultPeriodicBoxVectors(boxVectors[0], boxVectors[1], boxVectors[2]);
         double cutoff = owner.getCutoffDistance();
         if (cutoff > 0.5*boxVectors[0][0] || cutoff > 0.5*boxVectors[1][1] || cutoff > 0.5*boxVectors[2][2])
-            throw OpenMMException("GBSAOBCForce: The cutoff distance cannot be greater than half the periodic box size.");
+            throw OpenMMException("GBSAOBCForce: "+Messages::cutoffTooLarge);
     }
     for (int i = 0; i < owner.getNumParticles(); i++) {
         double charge, radius, scalingFactor;
