@@ -32,6 +32,7 @@
 #include "openmm/OpenMMException.h"
 #include "openmm/internal/ContextImpl.h"
 #include "openmm/internal/GayBerneForceImpl.h"
+#include "openmm/internal/Messages.h"
 #include "openmm/kernels.h"
 #include <set>
 #include <sstream>
@@ -115,7 +116,7 @@ void GayBerneForceImpl::initialize(ContextImpl& context) {
         system.getDefaultPeriodicBoxVectors(boxVectors[0], boxVectors[1], boxVectors[2]);
         double cutoff = owner.getCutoffDistance();
         if (cutoff > 0.5*boxVectors[0][0] || cutoff > 0.5*boxVectors[1][1] || cutoff > 0.5*boxVectors[2][2])
-            throw OpenMMException("GayBerneForce: The cutoff distance cannot be greater than half the periodic box size.");
+            throw OpenMMException("GayBerneForce: "+Messages::cutoffTooLarge);
     }
     kernel.getAs<CalcGayBerneForceKernel>().initialize(context.getSystem(), owner);
 }
