@@ -49,8 +49,8 @@ void MonteCarloAnisotropicBarostat::setDefaultPressure(const Vec3& pressure) {
 }
 
 void MonteCarloAnisotropicBarostat::setFrequency(int freq) {
-    if (freq <= 0)
-        throw OpenMMException("Frequency must be positive");
+    if (freq < 0)
+        throw OpenMMException("Frequency cannot be negative");
     frequency = freq;
 }
 
