@@ -44,8 +44,6 @@ MonteCarloMembraneBarostat::MonteCarloMembraneBarostat(double defaultPressure, d
 }
 
 void MonteCarloMembraneBarostat::setDefaultPressure(double pressure) {
-    if (pressure < 0)
-        throw OpenMMException("Pressure cannot be negative");
     defaultPressure = pressure;
 }
 
@@ -56,8 +54,8 @@ void MonteCarloMembraneBarostat::setDefaultSurfaceTension(double surfaceTension)
 }
 
 void MonteCarloMembraneBarostat::setFrequency(int freq) {
-    if (freq <= 0)
-        throw OpenMMException("Frequency must be positive");
+    if (freq < 0)
+        throw OpenMMException("Frequency cannot be negative");
     frequency = freq;
 }
 
