@@ -66,6 +66,7 @@ class PDBxFile(object):
         ## The Topology read from the PDBx/mmCIF file
         self.topology = top
         self._positions = []
+        PDBFile._loadNameReplacementTables()
 
         # Load the file.
 
@@ -152,7 +153,14 @@ class PDBxFile(object):
                     # The start of a new residue.
                     resId = (None if resNumCol == -1 else row[resNumCol])
                     resIC = insertionCode
-                    res = top.addResidue(row[resNameCol], chain, resId, resIC)
+                    resName = row[resNameCol]
+                    if resName in PDBFile._residueNameReplacements:
+                        resName = PDBFile._residueNameReplacements[resName]
+                    res = top.addResidue(resName, chain, resId, resIC)
+                    if resName in PDBFile._atomNameReplacements:
+                        atomReplacements = PDBFile._atomNameReplacements[resName]
+                    else:
+                        atomReplacements = {}
                     lastResId = row[resNumCol]
                     lastInsertionCode = insertionCode
                     atomsInResidue.clear()
@@ -161,9 +169,12 @@ class PDBxFile(object):
                     element = elem.get_by_symbol(row[elementCol])
                 except KeyError:
                     pass
-                atom = top.addAtom(row[atomNameCol], element, res, row[atomIdCol])
+                atomName = row[atomNameCol]
+                if atomName in atomReplacements:
+                    atomName = atomReplacements[atomName]
+                atom = top.addAtom(atomName, element, res, row[atomIdCol])
                 atomTable[atomKey] = atom
-                atomsInResidue.add(row[atomNameCol])
+                atomsInResidue.add(atomName)
             else:
                 # This row defines coordinates for an existing atom in one of the later models.
 
@@ -404,9 +415,9 @@ class PDBxFile(object):
         if is_quantity(positions):
             positions = positions.value_in_unit(angstroms)
         if any(math.isnan(norm(pos)) for pos in positions):
-            raise ValueError('Particle position is NaN')
+            raise ValueError('Particle position is NaN.  For more information, see https://github.com/openmm/openmm/wiki/Frequently-Asked-Questions#nan')
         if any(math.isinf(norm(pos)) for pos in positions):
-            raise ValueError('Particle position is infinite')
+            raise ValueError('Particle position is infinite.  For more information, see https://github.com/openmm/openmm/wiki/Frequently-Asked-Questions#nan')
         nonHeterogens = PDBFile._standardResidues[:]
         nonHeterogens.remove('HOH')
         atomIndex = 1
