@@ -466,6 +466,7 @@ class TestAPIUnits(unittest.TestCase):
         force.addPerParticleParameter('m')
         force.addParticle([1, 2, 3])
         force.addParticle([1*coulombs, 2*kilocalories_per_mole*angstroms**2, 3*kilocalories_per_mole*angstroms**3])
+        force.addTabulatedFunction('f', Continuous1DFunction([1,2,3,4,5], 0.0, 2.0))
 
         self.assertEqual(force.getNumParticles(), 2)
         charge, sigma, epsilon = force.getParticleParameters(0)
@@ -495,6 +496,8 @@ class TestAPIUnits(unittest.TestCase):
         force.setNonbondedMethod(CustomNonbondedForce.CutoffPeriodic)
         self.assertTrue(force.usesPeriodicBoundaryConditions())
 
+        self.assertIs(type(force.getTabulatedFunction(0)), Continuous1DFunction)
+
     def testCustomManyParticleForce(self):
         """ Tests the CustomManyParticleForce API features """
         force = CustomManyParticleForce(3,
@@ -1051,7 +1054,7 @@ class TestAPIUnits(unittest.TestCase):
         self.assertEqual(integrator.getStepSize(), 0.1*femtosecond)
         integrator.setStepSize(0.0005)
         self.assertEqual(integrator.getStepSize(), 0.0005*picosecond)
-        self.assertEqual(integrator.getMaxDrudeDistance(), 0*nanometer)
+        self.assertEqual(integrator.getMaxDrudeDistance(), 0.02*nanometer)
         integrator.setMaxDrudeDistance(0.05)
         self.assertEqual(integrator.getMaxDrudeDistance(), 0.05*nanometer)
 
